# coding=utf-8
class BaseFeature(object):
    def __init__(self, max_results=10):
        self.max_results = max_results


class LogoDetection(BaseFeature):
    type_ = "LOGO_DETECTION"


class LabelDetection(BaseFeature):
    type_ = "LABEL_DETECTION"


class UnspecifiedDetection(BaseFeature):
    type_ = "TYPE_UNSPECIFIED"


class FaceDetection(BaseFeature):
    type_ = "FACE_DETECTION"


class LandmarkDetection(BaseFeature):
    type_ = "LANDMARK_DETECTION"


class TextDetection(BaseFeature):
    type_ = "TEXT_DETECTION"


class SafeSearchDetection(BaseFeature):
    type_ = "SAFE_SEARCH_DETECTION"


class ImagePropertiesDetection(BaseFeature):
    type_ = "IMAGE_PROPERTIES"



# coding=utf-8
class GoogleCloudVisionException(Exception):
    def __init__(self, text):
        self.text = text

    def __unicode__(self):
        return "There was a problem with request: %s" % self.text



import json
from collections import namedtuple

Point = namedtuple('Point', ['x', 'y'])
BoundingPoly = namedtuple('BoundingPoly', ['vertices'])
BaseLogoAnnotation = namedtuple('LogoAnnotation', ['mid', 'description', 'score', 'boundingPoly'])
LabelAnnotation = namedtuple('LabelAnnotation', ['mid', 'description', 'score'])


# Fix default argument for namedtuple
def make_logo_annotation(mid, description, score, boundingPoly=None):
    return BaseLogoAnnotation(mid, description, score, boundingPoly)


class Response(object):
    def __init__(self, data):
        self.data = data

    @property
    def logo_annotations(self):
        responses = self.data.get('logoAnnotations', [])
        return [make_logo_annotation(**logo_annotation) for logo_annotation in responses]

    @property
    def label_annotations(self):
        responses = self.data.get('labelAnnotations', [])
        return [LabelAnnotation(**logo_annotation) for logo_annotation in responses]


class GoogleCloudVisionResponse(object):
    def __init__(self, content):
        self.content = content
        self.cache = None

    @property
    def error(self):
        return self.data.get('error')

    @property
    def ok(self):
        return self.data.get('error') is None

    @property
    def data(self):
        if self.cache is None:
            self.cache = json.loads(self.content)
        return self.cache

    @property
    def responses(self):
        responses = self.data.get('responses', [])
        return [Response(feature) for feature in responses]



# coding=utf-8
import json
import base64
import requests
from .exceptions import GoogleCloudVisionException
from .response import GoogleCloudVisionResponse


class GoogleCloudVision(object):
    API_URL_PATTERN = "https://vision.googleapis.com/v1/images:annotate?key=%s"
    HEADERS = {"Content-Type": "application/json"}

    def __init__(self, api_key):
        self.api_url = self.API_URL_PATTERN % api_key

    def make_request(self, image, *features):
        """
        Makes single image request
        :param image: One of file object, path, or URL
        :param features: Recognition features
        :return:
        """
        return {
            "image": {
                "content": self.image_to_base64(image)
            },
            "features": [{
                             "type": feature.type_,
                             "maxResults": feature.max_results
                         } for feature in features]
        }

    def image_to_base64(self, image):
        """
        :param image: One of file object, path, or URL
        :return: Base64 of image
        """
        if isinstance(image, file):
            return base64.b64encode(image.read())
        elif isinstance(image, str):
            if image.startswith("http"):
                # it's URL
                return base64.b64encode(requests.get(image).content)
            # it's path
            with open(image) as f:
                return base64.b64encode(f.read())

        raise ValueError("Unrecognizable image param: it must one of"
                         " file object, path or URL, not %s" % type(image))

    def make_body(self, image, *features):
        if isinstance(image, (list, tuple)) and len(image) > 1:
            requests_ = []
            for item in image:
                image_, features_ = item[0], item[1:]
                requests_.append(self.make_request(image_, *features_))
        else:
            requests_ = [self.make_request(image, *features)]
        return {'requests': requests_}

    def annotate(self, image, *features):
        """
        :param image: One of file object, path, or URL
        :param features: list of Vision Features
        :return: GoogleCloudVisionResponse
        """
        body = self.make_body(image, *features)
        try:
            response = requests.post(self.api_url, json.dumps(body), headers=self.HEADERS)
        except requests.RequestException as exc:
            raise GoogleCloudVisionException(exc)
        else:
            return GoogleCloudVisionResponse(response.content)



# -*- coding: utf-8 -*-
from distutils.core import setup
from setuptools import find_packages

VERSION = '1.0.0'

setup(
    name='visionary',
    description='Client library for Google Cloud Vision',
    url='https://github.com/shafaua/visionary',
    version=VERSION,
    author=u'Igor Gumenyuk',
    packages=find_packages(),
    install_requires=[
        'requests<3.0.0',
    ],
    include_package_data=True,
    zip_safe=False,
)
